import requests
import zstandard as zstd
import io
from datetime import datetime, timedelta
import os

def get_valid_date():
    today = datetime.now()
    first_day = today.replace(day=1)
    last_month = first_day - timedelta(days=1)
    max_year = last_month.year
    max_month = last_month.month

    while True:
        try:
            year = int(input(f"Enter year (2013-{max_year}): "))
            month = int(input(f"Enter month (1-12, up to {max_month} for {max_year}): "))
            
            if not (2013 <= year <= max_year):
                raise ValueError("Year out of range")
            if not (1 <= month <= 12):
                raise ValueError("Invalid month")
            if year == max_year and month > max_month:
                raise ValueError("Month out of range for current year")
            
            return year, month
        except ValueError as e:
            print(f"Invalid input: {e}. Please try again.")

def construct_url(year, month):
    return f"https://database.lichess.org/standard/lichess_db_standard_rated_{year}-{month:02d}.pgn.zst"

def download_file(url, filename):
    if os.path.exists(filename):
        print(f"File {filename} already exists. Skipping download.")
        return

    response = requests.get(url, stream=True)
    response.raise_for_status()
    
    total_size = int(response.headers.get('content-length', 0))
    downloaded = 0
    chunk_size = 1024 * 1024  # 1MB

    with open(filename, 'wb') as f:
        for chunk in response.iter_content(chunk_size=chunk_size):
            f.write(chunk)
            downloaded += len(chunk)
            print(f"Downloaded {downloaded / (1024**3):.2f} GB of {total_size / (1024**3):.2f} GB", end='\r')
    print("\nDownload completed.")

def get_rating_threshold():
    while True:
        try:
            rating = int(input("Enter minimum rating for both players: "))
            if rating < 0:
                raise ValueError("Rating cannot be negative")
            return rating
        except ValueError:
            print("Please enter a valid non-negative integer.")

def process_games(input_path, output_path, min_rating):
    dctx = zstd.ZstdDecompressor()
    
    with open(input_path, 'rb') as compressed_file:
        with dctx.stream_reader(compressed_file) as reader:
            text_reader = io.TextIOWrapper(reader, encoding='utf-8', errors='replace')
            
            with open(output_path, 'w', encoding='utf-8') as output_file:
                current_game = []
                white_elo = None
                black_elo = None
                in_headers = True

                for line in text_reader:
                    if line.startswith('[Event '):
                        if current_game:
                            if white_elo is not None and black_elo is not None and white_elo >= min_rating and black_elo >= min_rating:
                                output_file.writelines(current_game)
                                output_file.write('\n')
                            current_game = []
                            white_elo = None
                            black_elo = None
                            in_headers = True
                        current_game.append(line)
                    else:
                        current_game.append(line)
                        if in_headers:
                            stripped = line.strip()
                            if stripped.startswith('[WhiteElo "'):
                                try:
                                    white_elo = int(stripped.split('"')[1])
                                except (ValueError, IndexError):
                                    white_elo = None
                            elif stripped.startswith('[BlackElo "'):
                                try:
                                    black_elo = int(stripped.split('"')[1])
                                except (ValueError, IndexError):
                                    black_elo = None
                            elif stripped == '':
                                in_headers = False

                # Process last game
                if current_game and white_elo is not None and black_elo is not None and white_elo >= min_rating and black_elo >= min_rating:
                    output_file.writelines(current_game)

def main():
    print("Lichess PGN Filter Tool")
    print("-----------------------")
    
    year, month = get_valid_date()
    rating = get_rating_threshold()
    
    base_filename = f"lichess_db_standard_rated_{year}-{month:02d}"
    zst_filename = f"{base_filename}.pgn.zst"
    output_filename = f"{base_filename}_filtered_{rating}+.pgn"
    
    url = construct_url(year, month)
    
    print(f"\nDownloading {zst_filename}...")
    download_file(url, zst_filename)
    
    print("\nProcessing games...")
    process_games(zst_filename, output_filename, rating)
    
    print(f"\nFiltered games saved to {output_filename}")
    print("You can safely delete the .zst file now.")

if __name__ == "__main__":
    main()